/* ReSCAL - Main entry
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _MAIN_

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>

#include "defs.h"
#include "macros_public.h"
#include "entry.h"
#include "param.h"
#include "interface.h"
#include "callbacks.h"
#include "monitor.h"
#include "view.h"
#include "rescal.h"
#include "terre.h"

extern int graine;            // graine pour la generation des nombres aleatoires
#ifdef PARALLEL
extern int mode_par;    //mode parallele
#endif

gint area_w=0, area_h=0;
float frame_delay=0;
unsigned char opt_nv=0, opt_info=0, opt_dbin=0, opt_stop=0, opt_quit=0;
gint dbin_delay, stop_delay;
#ifdef LOG_FILE
FILE *log_file=NULL;
#endif

#ifdef DUMP_IMAGE
unsigned char opt_dpng=0, opt_djpeg=0;
gint dpng_delay, djpeg_delay;
#endif

int main (int argc, char *argv[])
{
  DumpPar dp_info, dp_bin, dp_png, dp_jpeg;

#ifdef LOG_FILE
  log_file = fopen("rescal.log","w");
#endif

  gtk_set_locale ();

  if ((argc == 1) || (*argv[1]=='-')){
    rescal_usage();
    param_usage();
    show_general_options();
    show_view_options();
    exit(-4);
  }

  general_options(argc, argv);

  if (!opt_nv){
    g_thread_init(NULL);
    gdk_threads_init();
    gtk_init (&argc, &argv);
  }
  /*else gtk_init_check (&argc, &argv);*/

  callbacks_init();
  rescal_init(argc, argv);
  view_init(argc, argv, &area_w, &area_h);
//  printf("coucou1\n");

  //if (opt_nv) rescal(0);

  if (!opt_nv) {
    GtkWidget *window = create_window();
    gtk_widget_show (window);
    gcallback_flash(window);
  }

  if (opt_info) {
    dump_info();
    dp_info = (DumpPar){DUMP_INFO, 1};
    g_timeout_add_seconds(60, gcallback_dump, &dp_info); //toutes les minutes
    //g_timeout_add_seconds(1, callback_dump, &dp_info); //toutes les secondes
  }

  if (opt_dbin) {
    dump_terre(dbin_delay);
    dp_bin = (DumpPar){DUMP_BIN, dbin_delay};
    g_timeout_add_seconds(dbin_delay*60, gcallback_dump, &dp_bin);
  }

#ifdef DUMP_IMAGE
  /*if (opt_dpng || opt_djpeg)*/ view_dump_init();
  if (opt_dpng) {
    view_dump_inter(dpng_delay, IMG_PNG);
    dp_png = (DumpPar){DUMP_PNG, dpng_delay};
    g_timeout_add_seconds(dpng_delay, gcallback_dump, &dp_png);
  }
  if (opt_djpeg) {
    view_dump_inter(djpeg_delay, IMG_JPEG);
    dp_jpeg = (DumpPar){DUMP_JPEG, djpeg_delay};
    g_timeout_add_seconds(djpeg_delay, gcallback_dump, &dp_jpeg);
  }
#endif

  if (opt_stop) {
    g_timeout_add_seconds_full(G_PRIORITY_DEFAULT, stop_delay*60+15, gcallback_stop, 0, NULL);
    //gtk_timeout_add(stop_delay*60000+15000, callback_stop, 0);
  }
  if (opt_quit){
    g_timeout_add_seconds_full(G_PRIORITY_DEFAULT, 1, gcallback_quit, 0, NULL);
  }
//  printf("coucou2\n"); fflush(stdout);

  int nRetVal;
  pthread_t pth;
  nRetVal = pthread_create( &pth, 0, rescal_thread, 0);

  /*
  GThread *pth;
  GError *err = NULL;
  g_thread_init(NULL);
  pth = g_thread_create((GThreadFunc)rescal_thread, 0, FALSE, &err);
  */

  //gtk_idle_add(idle, NULL);

  gtk_main ();

#ifdef LOG_FILE
  fclose(log_file);
#endif

  return 0;
}


void general_options(int argc, char *argv[])
{
  int i;
  for(i=1; i<argc; i++){
    if (!strcmp(argv[i],"-nv"))
      opt_nv = 1;
    else if (!strcmp(argv[i],"-fr")){
      int frame_rate = atoi(argv[++i]);
      frame_delay = (frame_rate<100) ? 1000/frame_rate : 10;
    }
    else if (!strcmp(argv[i],"-info")){
      opt_info = 1;
    }
    else if (!strcmp(argv[i],"-dbin")){
      opt_dbin = 1;
      dbin_delay = atoi(argv[++i]);
    }
#ifdef DUMP_IMAGE
    else if (!strcmp(argv[i],"-dpng")){
      opt_dpng = 1;
      dpng_delay = atoi(argv[++i]);
      if (!dpng_delay) dpng_delay = 1;
    }
    else if (!strcmp(argv[i],"-djpeg")){
      opt_djpeg = 1;
      djpeg_delay = atoi(argv[++i]);
      if (!djpeg_delay) djpeg_delay = 1;
    }
#endif
    else if (!strcmp(argv[i],"-stop")){
      opt_stop = 1;
      stop_delay = atoi(argv[++i]);
    }
    else if (!strcmp(argv[i],"-q")){
      opt_quit = 1;
    }
    else if (!strcmp(argv[i],"-g")){
      graine = atoi(argv[++i]);
    }
#ifdef PARALLEL
    else if (!strcmp(argv[i],"-par")){
      mode_par = 1;
    }
#endif
  }
}

void show_general_options()
{
  printf("\nGENERAL OPTIONS\n");
  printf("  -nv \t\t no window mode\n");
  printf("  -info \t monitoring data every minutes\n");
  printf("  -dbin <n> \t generation of a binary file every <n> minutes\n");
#ifdef DUMP_IMAGE
  printf("  -dpng <n> \t generation of a PNG image every <n> seconds\n");
  printf("  -djpeg <n> \t generation of a Jpeg image every <n> seconds\n");
#endif
  printf("  -stop <n> \t quit after <n> minutes\n");
  printf("  -q \t\t quit at the end of simulation\n");
  printf("  -fr <n> \t frame rate (fps)\n");
  printf("  -g <n> \t random seed\n");
//#ifdef MODELE_DUN
//  printf("  -ro <n> : rotation d'angle aleatoire toutes les <n> minutes\n");
//#endif
#ifdef PARALLEL
  printf("  -par \t\t parallel mode enabled\n");
  //printf("  -pars : mode parallele, process serveur\n");
  //printf("  -parc <i> : mode parallele, process client numero <i>\n");
#endif
  fflush(stdout);
}



