/* ReSCAL - Monitoring procedures
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>

#include "defs.h"
#include "macros_public.h"
#include "simul.h"
#include "terre.h"
#include "cellules.h"
#include "doublets.h"
#include "transitions.h"
#include "trace.h"

#ifdef COLLISIONS
#include "collisions.h"
#endif

#ifdef PARALLEL
#include "messages.h"
#include "synchro.h"
extern int mode_par;    //mode parallele
#endif

static char dump_ok = 0;

void monitor_signal(int);
void monitor_signal_term(int);
void monitor_signal_kill(int);

void start_dump()
{
  dump_ok = 1;
}

void stop_dump()
{
  dump_ok = 0;
}

void dump_info()
{
  if (dump_ok){
    dump_temps();
#ifdef INFO_CEL
    dump_cel_info();
#endif
#ifdef INFO_DBL
    dump_db_info();
#endif
#ifdef INFO_TRANS
    dump_trans_info();
    dump_check_info();
    dump_ln_info();
#endif
#ifdef PARALLEL
    if (mode_par) dump_msg_info();
#endif
#ifdef TRACE_TRANS
    trace_trans_dump();
#endif
#ifdef TRACE3D_CEL
    trace3d_cel_dump();
#endif
#ifdef COLLISIONS
  dump_densite();
  dump_vel();
#ifdef CGV
  dump_cgv_coef();
#endif
#endif
  }
}

void dump()
{
  if (dump_ok){
    dump_info();
    dump_terre(0);
#ifdef MODELE_AVA
    dump_surface(0);
#endif
#ifdef DUMP_RUGOSI
    dump_rugosi();
#endif
#if defined(COLLISIONS) && !defined(ROTATIONS)
    dump_mvt(0);
#endif
  }
  else
    LogPrintf("dump_ok = FALSE\n");
}

void init_signal()
{
  start_dump();
	/*signal(SIGUSR1,monitor_signal);
	signal(SIGTERM,monitor_signal_term);
	signal(SIGKILL,monitor_signal_kill);*/
}

void monitor_signal(int i)
{
  LogPrintf("monitor_signal()\n");
#ifdef PARALLEL
  if (mode_par) synchro_dump(); else dump();
#else
  dump();
#endif
}

void monitor_signal_term(int i)
{
  LogPrintf("signal term !\n");
	monitor_signal(i);
	exit(-1);
}

void monitor_signal_kill(int i)
{
  LogPrintf("signal kill !\n");
	monitor_signal(i);
	exit(-1);
}

int elapsed(double *sec)
{
  //extern int gettimeofday();
  struct timeval t;
  struct timezone tz;

  int stat;
  stat = gettimeofday(&t, &tz);
  *sec = (double)(t.tv_sec + t.tv_usec/1000000.0);
  return(stat);
}

